@@ -84,13 +84,6 @@ struct timeval
 
 #include <errno.h>
 
-#ifdef USE_LIBWRAP
-#include <syslog.h>
-#include <tcpd.h>
-int allow_severity=LOG_INFO;
-int deny_severity=LOG_WARNING;
-#endif
-
 /*#ifndef WIN32
 int max(int i,int j) { return(i<j?j:i); }
 #endif
@@ -104,6 +97,8 @@ int rfbMaxClientWait = 20000;   /* time (ms) after which we decide client has
  * connections.  It does nothing if called again.
  */
 
+static void rfbInitListenSock(rfbScreenInfoPtr rfbScreen);
+
 void
 rfbInitSockets(rfbScreenInfoPtr rfbScreen)
 {
@@ -134,12 +129,18 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
 	return;
     }
 
+    rfbInitListenSock(rfbScreen);
+}
+
+static void
+rfbInitListenSock(rfbScreenInfoPtr rfbScreen)
+{
     if(rfbScreen->autoPort) {
         int i;
         rfbLog("Autoprobing TCP port \n");
 
         for (i = 5900; i < 6000; i++) {
-            if ((rfbScreen->rfbListenSock = ListenOnTCPPort(i)) >= 0) {
+            if ((rfbScreen->rfbListenSock = ListenOnTCPPort(i, rfbScreen->localOnly)) >= 0) {
 		rfbScreen->rfbPort = i;
 		break;
 	    }
@@ -158,7 +159,7 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
     else if(rfbScreen->rfbPort>0) {
       rfbLog("Listening for VNC connections on TCP port %d\n", rfbScreen->rfbPort);
 
-      if ((rfbScreen->rfbListenSock = ListenOnTCPPort(rfbScreen->rfbPort)) < 0) {
+      if ((rfbScreen->rfbListenSock = ListenOnTCPPort(rfbScreen->rfbPort, rfbScreen->localOnly)) < 0) {
 	rfbLogPerror("ListenOnTCPPort");
 	return;
       }
@@ -169,16 +170,82 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
     }
 }
 
+void
+rfbSetAutoPort(rfbScreenInfoPtr rfbScreen, rfbBool autoPort)
+{
+    if (rfbScreen->autoPort == autoPort)
+        return;
+
+    rfbScreen->autoPort = autoPort;
+
+    if (!rfbScreen->socketInitDone)
+	return;
+
+    if (rfbScreen->rfbListenSock > 0) {
+        FD_CLR(rfbScreen->rfbListenSock, &(rfbScreen->allFds));
+        close(rfbScreen->rfbListenSock);
+        rfbScreen->rfbListenSock = -1;
+    }
+
+    rfbInitListenSock(rfbScreen);
+}
+
+void
+rfbSetPort(rfbScreenInfoPtr rfbScreen, int port)
+{
+    if (rfbScreen->rfbPort == port)
+        return;
+
+    rfbScreen->rfbPort = port;
+
+    if (!rfbScreen->socketInitDone || rfbScreen->autoPort)
+	return;
+
+    if (rfbScreen->rfbListenSock > 0) {
+        FD_CLR(rfbScreen->rfbListenSock, &(rfbScreen->allFds));
+        close(rfbScreen->rfbListenSock);
+        rfbScreen->rfbListenSock = -1;
+    }
+
+    rfbInitListenSock(rfbScreen);
+}
+
+void
+rfbSetLocalOnly(rfbScreenInfoPtr rfbScreen, rfbBool localOnly)
+{
+    if (rfbScreen->localOnly == localOnly)
+        return;
+
+    rfbScreen->localOnly = localOnly;
+
+    if (!rfbScreen->socketInitDone)
+	return;
+
+    if (rfbScreen->rfbListenSock > 0) {
+        FD_CLR(rfbScreen->rfbListenSock, &(rfbScreen->allFds));
+        close(rfbScreen->rfbListenSock);
+        rfbScreen->rfbListenSock = -1;
+    }
+
+    rfbLog("Re-binding socket to listen for %s VNC connections on TCP port %d\n",
+           rfbScreen->localOnly ? "local" : "all", rfbScreen->rfbPort);
+
+    if ((rfbScreen->rfbListenSock = ListenOnTCPPort(rfbScreen->rfbPort, rfbScreen->localOnly)) < 0) {
+	rfbLogPerror("ListenOnTCPPort");
+	return;
+    }
+
+    FD_SET(rfbScreen->rfbListenSock, &(rfbScreen->allFds));
+    rfbScreen->maxFd = max(rfbScreen->rfbListenSock, rfbScreen->maxFd);
+}
+
 void
 rfbProcessNewConnection(rfbScreenInfoPtr rfbScreen)
 {
-    struct sockaddr_in addr;
-    size_t addrlen = sizeof(addr);
     const int one = 1;
     int sock;
 
-    if ((sock = accept(rfbScreen->rfbListenSock,
-		       (struct sockaddr *)&addr, &addrlen)) < 0) {
+    if ((sock = accept(rfbScreen->rfbListenSock, NULL, NULL)) < 0) {
 	rfbLogPerror("rfbCheckFds: accept");
 	return;
     }
@@ -198,18 +265,6 @@ rfbProcessNewConnection(rfbScreenInfoPtr rfbScreen)
 	return;
     }
 
-#ifdef USE_LIBWRAP
-    if(!hosts_ctl("vnc",STRING_UNKNOWN,inet_ntoa(addr.sin_addr),
-		  STRING_UNKNOWN)) {
-	rfbLog("Rejected connection from client %s\n",
-	       inet_ntoa(addr.sin_addr));
-	close(sock);
-	return;
-    }
-#endif
-
-    rfbLog("Got connection from client %s\n", inet_ntoa(addr.sin_addr));
-
     rfbNewClient(rfbScreen,sock);
 }
 
@@ -287,54 +342,10 @@ rfbCloseClient(cl)
 	close(cl->sock);
 	cl->sock = -1;
       }
-    TSIGNAL(cl->updateCond);
     UNLOCK(cl->updateMutex);
 }
 
 
-/*
- * rfbConnect is called to make a connection out to a given TCP address.
- */
-
-int
-rfbConnect(rfbScreen, host, port)
-    rfbScreenInfoPtr rfbScreen;
-    char *host;
-    int port;
-{
-    int sock;
-    int one = 1;
-
-    rfbLog("Making connection to client on host %s port %d\n",
-	   host,port);
-
-    if ((sock = ConnectToTcpAddr(host, port)) < 0) {
-	rfbLogPerror("connection failed");
-	return -1;
-    }
-
-#ifndef WIN32
-    if (fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
-	rfbLogPerror("fcntl failed");
-	close(sock);
-	return -1;
-    }
-#endif
-
-    if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,
-		   (char *)&one, sizeof(one)) < 0) {
-	rfbLogPerror("setsockopt failed");
-	close(sock);
-	return -1;
-    }
-
-    /* AddEnabledDevice(sock); */
-    FD_SET(sock, &rfbScreen->allFds);
-    rfbScreen->maxFd = max(sock,rfbScreen->maxFd);
-
-    return sock;
-}
-
 #ifdef HAVE_GNUTLS
 static int
 ReadExactOverTLS(rfbClientPtr cl, char* buf, int len, int timeout)
@@ -537,28 +548,49 @@ WriteExact(rfbClientPtr cl, const char* buf, int len)
 }
 
 int
-ListenOnTCPPort(port)
+ListenOnTCPPort(port, localOnly)
     int port;
+    rfbBool localOnly;
 {
-    struct sockaddr_in addr;
-    int sock;
+    int sock = -1;
     int one = 1;
+    struct sockaddr_in addr_in;
+    struct sockaddr *addr;
+    socklen_t addrlen;
 
-    memset(&addr, 0, sizeof(addr));
-    addr.sin_family = AF_INET;
-    addr.sin_port = htons(port);
-    /* addr.sin_addr.s_addr = interface.s_addr; */
-    addr.sin_addr.s_addr = INADDR_ANY;
+#ifdef ENABLE_IPV6
+    struct sockaddr_in6 addr_in6;
 
-    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-	return -1;
+    memset(&addr_in6, 0, sizeof(addr_in6));
+    addr_in6.sin6_family = AF_INET6;
+    addr_in6.sin6_port = htons(port);
+    addr_in6.sin6_addr = localOnly ? in6addr_loopback : in6addr_any;
+
+    addr = (struct sockaddr *)&addr_in6;
+    addrlen = sizeof(addr_in6);
+
+    sock = socket(AF_INET6, SOCK_STREAM, 0);
+#endif
+
+    if (sock < 0) {
+        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+            return -1;
+
+        memset(&addr_in, 0, sizeof(addr_in));
+        addr_in.sin_family = AF_INET;
+        addr_in.sin_port = htons(port);
+        addr_in.sin_addr.s_addr = localOnly ? htonl(INADDR_LOOPBACK) : htonl(INADDR_ANY);
+
+        addr = (struct sockaddr *)&addr_in;
+        addrlen = sizeof(addr_in);
     }
+
     if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
 		   (char *)&one, sizeof(one)) < 0) {
 	close(sock);
 	return -1;
     }
-    if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+    if (bind(sock, addr, addrlen) < 0) {
 	close(sock);
 	return -1;
     }
@@ -569,37 +601,3 @@ ListenOnTCPPort(port)
 
     return sock;
 }
-
-int
-ConnectToTcpAddr(host, port)
-    char *host;
-    int port;
-{
-    struct hostent *hp;
-    int sock;
-    struct sockaddr_in addr;
-
-    memset(&addr, 0, sizeof(addr));
-    addr.sin_family = AF_INET;
-    addr.sin_port = htons(port);
-
-    if ((addr.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE)
-    {
-	if (!(hp = gethostbyname(host))) {
-	    errno = EINVAL;
-	    return -1;
-	}
-	addr.sin_addr.s_addr = *(unsigned long *)hp->h_addr;
-    }
-
-    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-	return -1;
-    }
-
-    if (connect(sock, (struct sockaddr *)&addr, (sizeof(addr))) < 0) {
-	close(sock);
-	return -1;
-    }
-
-    return sock;
-}
