@@ -36,6 +36,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #include <gdk/gdkx.h>
@@ -286,25 +287,21 @@ vino_fb_create_image (VinoFB           *vfb,
 
   if (!must_use_x_shm)
     {
-      int   rowstride = width * (depth / 8);
-      char *data;
-
-      data = malloc (rowstride * height);
-      if (!data)
-	return FALSE;
-
       *image = XCreateImage (vfb->priv->xdisplay,
 			     DefaultVisual (vfb->priv->xdisplay, 0),
 			     depth,
 			     ZPixmap,
 			     0,
-			     data,
+			     NULL,
 			     width,
 			     height,
-			     8,
-			     rowstride);
-      if (!*image)
-	free (data);
+			     32,
+			     0);
+      if (*image && !((*image)->data = malloc ((*image)->bytes_per_line * height)))
+        {
+          XDestroyImage(*image);
+          *image = NULL;
+        }
 
       return FALSE;
     }
@@ -582,17 +579,34 @@ vino_fb_xdamage_idle_handler (VinoFB *vfb)
   gdk_error_trap_push ();
 
   /* Copy the damaged pixels from the server */
-  XCopyArea (vfb->priv->xdisplay,
-	     GDK_WINDOW_XWINDOW (vfb->priv->root_window),
-	     vfb->priv->fb_pixmap,
-	     vfb->priv->xdamage_copy_gc,
-	     damage->x,
-	     damage->y,
-	     damage->width,
-	     damage->height,
-	     damage->x,
-	     damage->y);
-  XSync (vfb->priv->xdisplay, False);
+  if (vfb->priv->use_x_shm)
+    {
+      XCopyArea (vfb->priv->xdisplay,
+		 GDK_WINDOW_XWINDOW (vfb->priv->root_window),
+		 vfb->priv->fb_pixmap,
+		 vfb->priv->xdamage_copy_gc,
+		 damage->x,
+		 damage->y,
+		 damage->width,
+		 damage->height,
+		 damage->x,
+		 damage->y);
+      XSync (vfb->priv->xdisplay, False);
+    }
+  else
+    {
+      XGetSubImage (vfb->priv->xdisplay,
+		    GDK_WINDOW_XWINDOW (vfb->priv->root_window),
+		    damage->x,
+		    damage->y,
+		    damage->width,
+		    damage->height,
+		    AllPlanes,
+		    ZPixmap,
+		    vfb->priv->fb_image,
+		    damage->x,
+		    damage->y);
+    }
 
   if ((error = gdk_error_trap_pop ()))
     {
@@ -793,13 +807,16 @@ vino_fb_init_fb_image (VinoFB *vfb)
   if (vfb->priv->fb_image)
     {
 #ifdef HAVE_XSHM
-      vfb->priv->fb_pixmap = XShmCreatePixmap (vfb->priv->xdisplay,
-					       GDK_WINDOW_XWINDOW (vfb->priv->root_window),
-					       vfb->priv->fb_image->data,
-					       &vfb->priv->fb_image_x_shm_info,
-					       vfb->priv->fb_image->width,
-					       vfb->priv->fb_image->height,
-					       vfb->priv->fb_image->depth);
+      if (vfb->priv->use_x_shm)
+	{
+	  vfb->priv->fb_pixmap = XShmCreatePixmap (vfb->priv->xdisplay,
+						   GDK_WINDOW_XWINDOW (vfb->priv->root_window),
+						   vfb->priv->fb_image->data,
+						   &vfb->priv->fb_image_x_shm_info,
+						   vfb->priv->fb_image->width,
+						   vfb->priv->fb_image->height,
+						   vfb->priv->fb_image->depth);
+	}
 #endif
       if (vfb->priv->fb_pixmap == None)
 	{
@@ -838,6 +855,15 @@ vino_fb_init_from_screen (VinoFB    *vfb,
 
 #ifdef HAVE_XSHM
   vfb->priv->use_x_shm = XShmQueryExtension (vfb->priv->xdisplay) != False;
+  if (vfb->priv->use_x_shm)
+    {
+      int major, minor;
+      Bool shared_pixmaps;
+
+      XShmQueryVersion (vfb->priv->xdisplay, &major, &minor, &shared_pixmaps);
+      if (!shared_pixmaps)
+	vfb->priv->use_x_shm = FALSE;
+    }
 #endif
 
   g_signal_connect_swapped (vfb->priv->screen, "size-changed",
@@ -951,10 +977,14 @@ vino_fb_class_init (VinoFBClass *klass)
   g_object_class_install_property (gobject_class,
 				   PROP_SCREEN,
 				   g_param_spec_object ("screen",
-							_("Screen"),
-							_("The screen to be monitored"),
+							"Screen",
+							"The screen to be monitored",
 							GDK_TYPE_SCREEN,
-							G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+							G_PARAM_READWRITE      |
+                                                        G_PARAM_CONSTRUCT_ONLY |
+                                                        G_PARAM_STATIC_NAME    |
+                                                        G_PARAM_STATIC_NICK    |
+                                                        G_PARAM_STATIC_BLURB));
 
   signals [DAMAGE_NOTIFY] =
     g_signal_new ("damage-notify",
