@@ -47,7 +47,7 @@
 
 /* See <X11/keysymdef.h> - "Latin 1: Byte 3 and 4 = 0"
  */
-#define VINO_IS_LATIN1_KEYSYM(k) ((k) != NoSymbol && ((k) & 0xff00) == 0)
+#define VINO_IS_LATIN1_KEYSYM(k) ((k) != NoSymbol && ((k) & 0xffffff00) == 0)
 
 #define VINO_IS_MODIFIER_KEYSYM(k) (((k) >= XK_Shift_L && (k) <= XK_Hyper_R) || \
                                      (k) == XK_Mode_switch                   || \
